//
//  Image.m
//  An abstract representation of an image, with the metadata
//
//  Created by Alexis Taugeron on 21/11/10.
//  Copyright 2010 Alexis Taugeron & Adrien Kohlbecker
//
//	Licensed under the Apache License, Version 2.0 (the "License");
//	you may not use this file except in compliance with the License.
//	You may obtain a copy of the License at
//	
//	http://www.apache.org/licenses/LICENSE-2.0
//	
//	Unless required by applicable law or agreed to in writing, software
//	distributed under the License is distributed on an "AS IS" BASIS,
//	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//	See the License for the specific language governing permissions and
//	limitations under the License.
//	

#import "Image.h"

@interface Image ()

- (NSString *)metadataPath;
- (BOOL)retrieveMetadataFromCache;

@end


@implementation Image

@synthesize identifier,ready;


#pragma mark -
#pragma mark Initialization

- (id)initWithID:(NSString *)anID source:(id <ImageSource>)aSource {
	if ((self = [super init])) {
		//Retain the parameters
		identifier = [anID retain];
		source = [(NSObject *)aSource retain];
		
		//We try to retrieve the metadata from the cache (it will work if the image has been cached to save memory)
		//If it's the first time we load this image, we ask the source for the metadata
		if (![self retrieveMetadataFromCache]) {
			[source requestMetadataForSender:self];
		} else {
			[self setValue:[NSNumber numberWithBool:YES] forKey:@"ready"];
		}
	}
	
	return self;
}


#pragma mark -
#pragma mark Metadata accessors

- (NSArray *)getMetadataList {
	if (metadata == nil) [self retrieveMetadataFromCache];
	return [metadata allKeys];
}

- (id)valueForMetadata:(NSString *)metadataName {
	if (metadata == nil) [self retrieveMetadataFromCache];
	return [metadata valueForKey:metadataName];
}

- (void)setValue:(id)value forMetadata:(NSString *)metadataName {
	if (metadata == nil) [self retrieveMetadataFromCache];
	[metadata setValue:value forKey:metadataName];
}


#pragma mark -
#pragma mark Visual representations accessors

- (void)requestFullSizeRepresentationForSender:(id <ImageManager>)sender {
	[source requestFullSizeRepresentationWithID:identifier forSender:sender];
}

- (void)requestThumbnailRepresentationOfSize:(NSUInteger)size sender:(id <ImageManager>)sender; {
	[source requestThumbnailRepresentationOfSize:size ID:identifier sender:sender];
}


#pragma mark -
#pragma mark ImageSourceMetadataDelegate protocol

- (void)metadataReady:(NSDictionary *)dictionary {
	metadata = [dictionary retain];
	[metadata writeToFile:[self metadataPath] atomically:YES];
	[self setValue:[NSNumber numberWithBool:YES] forKey:@"ready"];
}

#pragma mark -
#pragma mark Cache management

- (NSString *)metadataPath {
	return [NSString stringWithFormat:@"%@%@-metadata",NSTemporaryDirectory(),identifier];
}

- (BOOL)retrieveMetadataFromCache {
	if ([[NSFileManager defaultManager] fileExistsAtPath:[self metadataPath]]) {
		metadata = [[NSDictionary alloc] initWithContentsOfFile:[self metadataPath]];
		return YES;
	}
	return NO;
}


#pragma mark -
#pragma mark Key-value tricks

- (void)checkReady {
	[self setValue:[NSNumber numberWithBool:ready] forKey:@"ready"];
}


#pragma mark -
#pragma mark Memory management

- (void)cleanMemory {
	[metadata release];
	metadata = nil;
}

- (void)dealloc {
	[identifier release];
	[(NSObject *)source release];
	[metadata release];
	
	[super dealloc];
}

@end
