//
//  ImageDatabase.m
//  The object representation of the image database
//
//  Created by Alexis Taugeron on 21/11/10.
//  Copyright 2010 Alexis Taugeron & Adrien Kohlbecker
//
//	Licensed under the Apache License, Version 2.0 (the "License");
//	you may not use this file except in compliance with the License.
//	You may obtain a copy of the License at
//	
//	http://www.apache.org/licenses/LICENSE-2.0
//	
//	Unless required by applicable law or agreed to in writing, software
//	distributed under the License is distributed on an "AS IS" BASIS,
//	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//	See the License for the specific language governing permissions and
//	limitations under the License.
//

#import "ImageDatabase.h"


@implementation ImageDatabase

@synthesize imagesReady, ready;


#pragma mark -
#pragma mark Initialization

- (id)initWithSource:(id <ImageSource>)aSource {
	if ((self = [super init])) {
		//Retain the parameter
		source = [(NSObject *)aSource retain];
		
		//Set the images mutable dictionary
		images = [[NSMutableDictionary alloc] init];
		
		//Ask the source for the identifiers
		[source requestIdentifiersListForSender:self];
	}
	
	return self;
}


#pragma mark -
#pragma mark Accessors

- (NSUInteger)countOfIdentifiers {
	return [identifiers count];
}

- (NSString *)identifierAtIndex:(NSUInteger)index {
	return [identifiers objectAtIndex:index];
}

- (void)requestImageAtIndex:(NSUInteger)index sender:(id <ImageDatabaseManager>)sender {
	//If the image is in the memory, we just return it
	//Otherwise (and it means it has been cached), we recreate it
	if ([[images allKeys] containsObject:[identifiers objectAtIndex:index]]) {
		[sender imageReady:(Image *)[images objectForKey:[identifiers objectAtIndex:index]] forIndex:index];
	} else {
		Image *image = [[Image alloc] initWithID:[identifiers objectAtIndex:index] source:source];
		[sender imageReady:[image autorelease] forIndex:index];
	}
}


#pragma mark -
#pragma mark ImageSourceListDelegate protocol

- (void)identifiersListReady:(NSArray *)list {
	//Retain the parameter
	identifiers = [list retain];
	
	//Create all the images and wait for them to be ready before considering the database ready (see the KVO method)
	imagesReady = 0;
	for (NSString *identifier in identifiers) {
		Image *image = [[Image alloc] initWithID:identifier source:source];
		[image addObserver:self forKeyPath:@"ready" options:NSKeyValueObservingOptionNew context:NULL];
		[image checkReady];
		[images setValue:[image autorelease] forKey:identifier];
	}
}


#pragma mark -
#pragma mark KVO

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context {
	if ([keyPath isEqual:@"ready"]) {
		//If we are notified that an image is ready, we increment a counter.
		//If the counter reach to total number of images, the database is ready.
		if (ready == NO) {
			imagesReady++;
			
			if (imagesReady == self.countOfIdentifiers) [self setValue:[NSNumber numberWithBool:YES] forKey:@"ready"];
		}
	}
}


#pragma mark -
#pragma mark Key-value tricks

- (void)checkReady {
	[self setValue:[NSNumber numberWithBool:ready] forKey:@"ready"];
}


#pragma mark -
#pragma mark Memory management

- (void)cleanMemory {
	[images removeAllObjects];
}

- (void)dealloc {
	[(NSObject *)source release];
	[identifiers release];
	[images release];
	
	[super dealloc];
}

@end
