//
//  ImageGridController.m
//  The area of the screen where we display the thumbnails or full-size images
//
//  Created by Alexis Taugeron on 21/11/10.
//  Copyright 2010 Alexis Taugeron & Adrien Kohlbecker
//
//	Licensed under the Apache License, Version 2.0 (the "License");
//	you may not use this file except in compliance with the License.
//	You may obtain a copy of the License at
//	
//	http://www.apache.org/licenses/LICENSE-2.0
//	
//	Unless required by applicable law or agreed to in writing, software
//	distributed under the License is distributed on an "AS IS" BASIS,
//	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//	See the License for the specific language governing permissions and
//	limitations under the License.
//

#import "ImageGridController.h"

@interface ImageGridController ()

@property (readonly) NSUInteger bordersSize;
@property (readonly) NSUInteger countOfLines;
@property (readonly) NSUInteger countOfImagesPerLine;

- (void)gridWillChangeWidthWithInterfaceOrientation:(UIInterfaceOrientation)orientation duration:(NSTimeInterval)duration;
- (void)updateScrollViewContentSize;
- (void)buildGrid;
- (void)updateFramesWithAnimationDuration:(NSTimeInterval)duration;
- (CGRect)frameForIndex:(NSUInteger)index;
- (void)handleTap:(UIGestureRecognizer *)gestureRecognizer;
- (void)handlePinch:(UIGestureRecognizer *)gestureRecognizer;

@end


@implementation ImageGridController

@synthesize delegate,fullScreen;


#pragma mark -
#pragma mark Initialization

- (id)initWithDatabase:(ImageDatabase *)db {
	if ((self = [super initWithNibName:nil bundle:nil])) {
		//Make the scroll view
		UIScrollView *scrollView = [[[UIScrollView alloc] initWithFrame:self.view.frame] autorelease];
		scrollView.backgroundColor = PHOTOTYPE_GRID_BACKGROUND_COLOR;
		scrollView.alwaysBounceVertical = YES;
		self.view = [scrollView autorelease];
		[self updateScrollViewContentSize];
		
		//Bind a pinch gesture recognizer to the view
		UIPinchGestureRecognizer *pinchGR = [[UIPinchGestureRecognizer alloc] initWithTarget:self action:@selector(handlePinch:)];
		[self.view addGestureRecognizer:pinchGR];
		[pinchGR release];
		
		//Initialize the width of the grid
		fullScreen = YES;
		intOrientation = self.interfaceOrientation;
		[self gridWillChangeWidthWithInterfaceOrientation:intOrientation duration:0];
		
		//Retain the parameter
		database = [db retain];
		
		//Initialize the dictionary where we will store the image views
		imageViews = [[NSMutableDictionary alloc] init];
		
		//Display the loading view
		loadingView = [[DatabaseLoadingView alloc] initWithFrame:self.view.bounds];
		[self.view addSubview:loadingView];
		
		//Register for database isReady notification
		[database addObserver:self forKeyPath:@"ready" options:NSKeyValueObservingOptionNew context:NULL];
		[database checkReady];
	}
	
	return self;
}


#pragma mark -
#pragma mark Accessors

- (NSUInteger)bordersSize {
	//Calculate an automatic border size
	return ceil((float)(widthOfGrid - self.countOfImagesPerLine*PHOTOTYPE_THUMBNAILS_SIZE) / ((float)(self.countOfImagesPerLine+1)));
}

- (NSUInteger)countOfLines {
	//Calculate the min number of lines
	NSUInteger count = ceil((float)database.countOfIdentifiers / (float)self.countOfImagesPerLine);
	
	return count;
}

- (NSUInteger)countOfImagesPerLine {
	//Calculate the max number of images per line
	NSUInteger count = floor(widthOfGrid / (float)(PHOTOTYPE_THUMBNAILS_SIZE*(1+PHOTOTYPE_GRID_MINIMUM_BORDER_RATIO)));
	
	return count;
}

- (CGRect)frameForIndex:(NSUInteger)index {
	//Calculate the position in the grid
	NSUInteger lineCoord = floor((float)index / (float)self.countOfImagesPerLine);
	NSUInteger columnCoord = index - self.countOfImagesPerLine*lineCoord;
	
	CGFloat x = self.bordersSize + columnCoord * (self.bordersSize + PHOTOTYPE_THUMBNAILS_SIZE);
	CGFloat y = self.bordersSize + lineCoord * (self.bordersSize + PHOTOTYPE_THUMBNAILS_SIZE);
	CGFloat width = PHOTOTYPE_THUMBNAILS_SIZE, height = PHOTOTYPE_THUMBNAILS_SIZE;
	return CGRectMake(x, y, width, height);
}

- (void)setFullScreen:(BOOL)value {
	fullScreen = value;
	[self gridWillChangeWidthWithInterfaceOrientation:intOrientation duration:PHOTOTYPE_SPLIT_TOGGLE_DURATION];
}


#pragma mark -
#pragma mark UI syncronization

- (void)gridWillChangeWidthWithInterfaceOrientation:(UIInterfaceOrientation)orientation duration:(NSTimeInterval)duration {
	//Update the width of the grid
	if (UIInterfaceOrientationIsPortrait(orientation)) {
		widthOfGrid = PHOTOTYPE_SCREEN_PORTRAIT_WIDTH;
	} else {
		widthOfGrid = PHOTOTYPE_SCREEN_LANDSCAPE_WIDTH;
		if (!fullScreen) widthOfGrid -= PHOTOTYPE_FILTER_PANEL_LANDSCAPE_WIDTH + PHOTOTYPE_SPLIT_WIDTH;
	}
	
	//DEBUG LINES
	NSLog(@"Grid has changed width : %u", widthOfGrid);
	//-DEBUG LINES
	
	[self updateScrollViewContentSize];
	[self updateFramesWithAnimationDuration:duration];
	[self buildGrid];
}

- (void)updateScrollViewContentSize {
	//Calculate the content size
	CGFloat width = widthOfGrid;
	CGFloat height = (CGFloat)((PHOTOTYPE_THUMBNAILS_SIZE + self.bordersSize) * self.countOfLines + self.bordersSize);
	CGSize size = CGSizeMake(width, height);
	
	//DEBUG LINES
	NSLog(@"Updated scroll view content size. Width : %u - Height : %u",(NSUInteger)width,(NSUInteger)height);
	//-DEBUG LINES
	
	UIScrollView *scrollView = (UIScrollView *)self.view;
	scrollView.contentSize = size;
}

- (void)buildGrid {
	//Iterate on the lines, then on the columns
	NSUInteger lineCounter,columnCounter,queueCounter=0;
	NSOperationQueue *queue = nil;
	for (lineCounter = 0 ; lineCounter < self.countOfLines ; lineCounter++) {
		for (columnCounter = 0 ; columnCounter < self.countOfImagesPerLine ; columnCounter++) {
			//Calculate the index. If it's beyond the size of the database, we skip this step.
			NSUInteger index = lineCounter * self.countOfImagesPerLine + columnCounter;
			if (index < database.countOfIdentifiers) {
				//Calculate the thumbnail position
				CGRect frame = [self frameForIndex:index];
				
				//If the thumbnail isn't already displayed, we make it.
				//Otherwise, we rely on other methods to update its frame if necessary
				if (![[imageViews allKeys] containsObject:[database identifierAtIndex:index]]) {
					//Make the thumbnail and add a tap gesture recognizer to it
					UIImageView *imageView = [[UIImageView alloc] initWithFrame:frame];
					imageView.userInteractionEnabled = YES;
					UITapGestureRecognizer *tapGR = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(handleTap:)];
					[imageView addGestureRecognizer:tapGR];
					[tapGR release];
					[self.view addSubview:imageView];
					[imageViews setObject:imageView forKey:[database identifierAtIndex:index]];
					[imageView release];
					
					//Requesting the thumbnail data in background (we pack the operations ten by ten)
					if (queue == nil) queue = [[NSOperationQueue alloc] init];
					if (queueCounter >= 10) {
						[queue autorelease];
						queue = [[NSOperationQueue alloc] init];
						queueCounter = 0;
					}
					
					[queue addOperationWithBlock:^{
						[database requestImageAtIndex:index sender:self];
					}];
					
					queueCounter++;
				}
			}
		}
	}
	
	[queue autorelease];
}

- (void)updateFramesWithAnimationDuration:(NSTimeInterval)duration {
	//Iterate on the displayed image view to update their frames
	NSUInteger counter;
	for (counter = 0 ; counter < database.countOfIdentifiers ; counter++) {
		if ([[imageViews allKeys] containsObject:[database identifierAtIndex:counter]]) {
			//Retrieve the image view
			UIImageView *imageView = (UIImageView *)[imageViews objectForKey:[database identifierAtIndex:counter]];
			
			//Make the animation
			[UIView beginAnimations:nil context:NULL];
			[UIView setAnimationDuration:duration];
			imageView.frame = [self frameForIndex:counter];
			[UIView commitAnimations];
		}
	}
}


#pragma mark -
#pragma mark Rotation

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    return YES;
}

- (void)willRotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration {
	//Update the width of the grid
	intOrientation = toInterfaceOrientation;
	[self gridWillChangeWidthWithInterfaceOrientation:toInterfaceOrientation duration:duration];
}


#pragma mark -
#pragma mark ImageManager protocol

- (void)representationReady:(UIImage *)representation forImage:(NSString *)identifier {
	//DEBUG LINES
	NSLog(@"Representation of size %fx%f ready for identifier %@",representation.size.width,representation.size.height,identifier);
	//-DEBUG LINES
	
	//Get the corresponding image view
	UIImageView *imageView = (UIImageView *)[imageViews objectForKey:identifier];
		
	//Animate the transition
	CGRect frame = imageView.frame;
	CGSize size = frame.size;
	CGPoint center = imageView.center;
	frame.size = CGSizeZero;
	imageView.frame = frame;
	imageView.center = center;
	[UIView beginAnimations:nil context:NULL];
	[UIView setAnimationDuration:PHOTOTYPE_THUMBNAILS_FADE_DURATION];
	imageView.image = representation;
	frame.size = size;
	imageView.frame = frame;
	[UIView commitAnimations];
}


#pragma mark -
#pragma mark ImageDatabaseManager protocol

- (void)imageReady:(Image *)image forIndex:(NSUInteger)index {
	[image requestThumbnailRepresentationOfSize:PHOTOTYPE_THUMBNAILS_SIZE sender:self];
}


#pragma mark -
#pragma mark KVO

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context {
	if ([keyPath isEqual:@"ready"]) {
		//When the database is ready, we remove the loading view and display the grid
		if ([self.view.subviews containsObject:loadingView]) {
			[loadingView removeFromSuperview];
			[self updateScrollViewContentSize];
			[self buildGrid];
		}
	}
}


#pragma mark -
#pragma mark Gesture recognizers

- (void)handleTap:(UIGestureRecognizer *)gestureRecognizer {
	//DEBUG LINES
	NSLog(@"Tap detected");
	//-DEBUG LINES
}

- (void)handlePinch:(UIGestureRecognizer *)gestureRecognizer {
	//DEBUG LINES
	NSLog(@"Pinch detected");
	//-DEBUG LINES
}


#pragma mark -
#pragma mark Memory management

- (void)dealloc {
	[delegate retain];
	[database release];
	[loadingView release];
	[imageViews release];
	
    [super dealloc];
}


@end
